﻿using System;
using System.Net;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace GridAnimation
{
  public class MaximizeRestoreAnimation : Timeline
  {
    public static readonly DependencyProperty StoryboardProperty =
    DependencyProperty.Register("Storyboard",
        typeof(Storyboard),
        typeof(MaximizeRestoreAnimation),
        new PropertyMetadata(OnStoryboardChanged));

    public Storyboard Storyboard
    {
      set { SetValue(StoryboardProperty, value); }
      get { return (Storyboard)GetValue(StoryboardProperty); }
    }

    public MainPage MainPage { get; set; }

    static void OnStoryboardChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
    {
      var thisAnimation = obj as MaximizeRestoreAnimation;
      if (thisAnimation != null)
      {
        thisAnimation.Storyboard.Completed += thisAnimation.MaximizeRestoreAnimation_Completed;
        CompositionTarget.Rendering += thisAnimation.OnCompositionTargetRendering;
      }
    }

    /// <summary>
    /// Finalize layout after animation has completed.
    /// </summary>
    void MaximizeRestoreAnimation_Completed(object sender, EventArgs e)
    {
      _running = false;
      if (_maximized)
        EndMaximize();
      else
        EndRestore();
    }

    void OnCompositionTargetRendering(object sender, EventArgs e)
    {
      // Comes here for every rendered frame, so don't waste performance if nothing is to animate
      if (!(_startAnimation || _running))
        return;

      // Prevent division by zero
      if (!Duration.HasTimeSpan)
        return;

      // Calculate progress using the storyboard Ticks
      double progress = (double)Storyboard.GetCurrentTime().Ticks / Duration.TimeSpan.Ticks;

      if (_startAnimation)
      {
        // switch to running
        _startAnimation = false;
        _running = true;
      }

      if (_maximized)
        AnimateMaximize(progress);
      else
        AnimateRestore(progress);
    }

    public void ToggleMaximized()
    {
      _maximized = !_maximized;
      _startAnimation = true;
      if (_maximized)
        StartMaximize();
      else
        StartRestore();
      Storyboard.Begin();
    }

    void StartMaximize()
    {
      _lrc0Width = MainPage.LayoutRoot.ColumnDefinitions[0].ActualWidth;
      _lrc2Width = MainPage.LayoutRoot.ColumnDefinitions[2].ActualWidth;

      MainPage.LayoutRoot.ColumnDefinitions[0].Width = new GridLength(_lrc0Width);
      MainPage.LayoutRoot.ColumnDefinitions[1].Width = new GridLength(1, GridUnitType.Star);
      MainPage.LayoutRoot.ColumnDefinitions[2].Width = new GridLength(_lrc2Width);

      MainPage.InnerLayout.RowDefinitions[0].Height = new GridLength(100);
      MainPage.InnerLayout.RowDefinitions[1].Height = new GridLength(1, GridUnitType.Star);
      MainPage.InnerLayout.RowDefinitions[2].Height = new GridLength(30);
      MainPage.InnerLayout.ColumnDefinitions[0].Width = new GridLength(180);
      MainPage.InnerLayout.ColumnDefinitions[1].Width = new GridLength(1, GridUnitType.Star);
    }

    void AnimateMaximize(double progress)
    {
      var factor = 1 - new PowerEase { EasingMode = EasingMode.EaseOut, Power = 3 }.Ease(progress);

      MainPage.LayoutRoot.ColumnDefinitions[0].Width = new GridLength(factor * _lrc0Width);
      MainPage.LayoutRoot.ColumnDefinitions[2].Width = new GridLength(factor * _lrc2Width);

      MainPage.InnerLayout.RowDefinitions[0].Height = new GridLength(factor * 100);
      MainPage.InnerLayout.RowDefinitions[2].Height = new GridLength(factor * 30);
      MainPage.InnerLayout.ColumnDefinitions[0].Width = new GridLength(factor * 180);
    }

    void EndMaximize()
    {
      // not required - just for completeness

      MainPage.LayoutRoot.ColumnDefinitions[0].Width = new GridLength(zero, GridUnitType.Pixel);
      MainPage.LayoutRoot.ColumnDefinitions[1].Width = new GridLength(1, GridUnitType.Star);
      MainPage.LayoutRoot.ColumnDefinitions[2].Width = new GridLength(zero, GridUnitType.Pixel);

      MainPage.InnerLayout.RowDefinitions[0].Height = new GridLength(zero, GridUnitType.Pixel);
      MainPage.InnerLayout.RowDefinitions[1].Height = new GridLength(1, GridUnitType.Star);
      MainPage.InnerLayout.RowDefinitions[2].Height = new GridLength(zero, GridUnitType.Pixel);
      MainPage.InnerLayout.ColumnDefinitions[0].Width = new GridLength(zero, GridUnitType.Pixel);
      MainPage.InnerLayout.ColumnDefinitions[1].Width = new GridLength(1, GridUnitType.Star);
    }

    void StartRestore()
    {
      double width = MainPage.ActualWidth;
      _lrc0Width = _lrc2Width = (width - 700) / 2;

      MainPage.LayoutRoot.ColumnDefinitions[0].Width = new GridLength(zero);
      MainPage.LayoutRoot.ColumnDefinitions[1].Width = new GridLength(1, GridUnitType.Star);
      MainPage.LayoutRoot.ColumnDefinitions[2].Width = new GridLength(zero);

      MainPage.InnerLayout.RowDefinitions[0].Height = new GridLength(zero);
      MainPage.InnerLayout.RowDefinitions[1].Height = new GridLength(1, GridUnitType.Star);
      MainPage.InnerLayout.RowDefinitions[2].Height = new GridLength(zero);
      MainPage.InnerLayout.ColumnDefinitions[0].Width = new GridLength(zero);
      MainPage.InnerLayout.ColumnDefinitions[1].Width = new GridLength(1, GridUnitType.Star);
    }

    void AnimateRestore(double progress)
    {
      var factor = new PowerEase { EasingMode = EasingMode.EaseOut, Power = 3 }.Ease(progress);

      MainPage.LayoutRoot.ColumnDefinitions[0].Width = new GridLength(factor * _lrc0Width);
      MainPage.LayoutRoot.ColumnDefinitions[2].Width = new GridLength(factor * _lrc2Width);

      MainPage.InnerLayout.RowDefinitions[0].Height = new GridLength(factor * 100);
      MainPage.InnerLayout.RowDefinitions[2].Height = new GridLength(factor * 30);
      MainPage.InnerLayout.ColumnDefinitions[0].Width = new GridLength(factor * 180);
    }

    void EndRestore()
    {
      MainPage.LayoutRoot.ColumnDefinitions[0].Width = new GridLength(1, GridUnitType.Star);
      MainPage.LayoutRoot.ColumnDefinitions[1].Width = new GridLength(700, GridUnitType.Pixel);
      MainPage.LayoutRoot.ColumnDefinitions[2].Width = new GridLength(1, GridUnitType.Star);

      MainPage.InnerLayout.RowDefinitions[0].Height = new GridLength(100, GridUnitType.Pixel);
      MainPage.InnerLayout.RowDefinitions[1].Height = new GridLength(1, GridUnitType.Star);
      MainPage.InnerLayout.RowDefinitions[2].Height = new GridLength(30, GridUnitType.Pixel);
      MainPage.InnerLayout.ColumnDefinitions[0].Width = new GridLength(180, GridUnitType.Pixel);
      MainPage.InnerLayout.ColumnDefinitions[1].Width = new GridLength(1, GridUnitType.Star);
    }

    const int zero = 0;  // set to non-zero for demonstration purposes

    bool _maximized;
    bool _startAnimation;
    bool _running;

    double _lrc0Width;
    double _lrc2Width;
  }
}